from sys import version_info, exit
from setuptools import setup, find_packages
from os import path

if version_info[0] < 3 and version_info[1] < 5:
    exit("Sorry, support only for Python 3.5 and above.")

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="tornado-razorpay",
    version="0.1.3",
    description="Razorpay Asynchronous Tornado Python Client",
    url="https://github.com/nkanish2002/tornado-razorpay",
    author="Anish Gupta",
    author_email="nkanish2002@gmail.com",
    license="MIT",
    install_requires=["tornado>4"],
    package_dir={'tornado_razorpay': 'tornado_razorpay'},
    packages=find_packages(),
    keywords='razorpay payment gateway india tornado async',
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ]
)



from tornado.httpclient import AsyncHTTPClient


def _optional_args(args_lst):
    args_str = "?"
    for args in args_lst:
        if args[1] is not None:
            args_str = "%s%s=%s&" % (args_str, args[0], str(args[1]))
    return args_str


class _Payments:
    def __init__(self, api_key, api_secret, client: AsyncHTTPClient):
        self.api_key = api_key
        self.api_secret = api_secret
        self.base_url = "https://api.razorpay.com/"
        self.version = "v1"
        self.payment_directory = "payments"
        self.api_url = "%s%s/%s/" % (self.base_url, self.version, self.payment_directory)
        self.http_client = client

    async def all(self, from_time=None, to_time=None, count=None, skip=None):
        args = _optional_args([["from", from_time], ["to", to_time], ["count", count], ["skip", skip]])
        url = "%s%s" % (self.api_url, args)
        response = await self.http_client.fetch(url, auth_username=self.api_key, auth_password=self.api_secret,
                                                raise_error=False)
        return response

    async def fetch(self, payment_id):
        url = "%s%s" % (self.api_url, payment_id)
        response = await self.http_client.fetch(url, method="GET", auth_username=self.api_key,
                                                auth_password=self.api_secret, raise_error=False)
        return response

    async def capture(self, payment_id, amount):
        url = "%s%s/capture" % (self.api_url, payment_id)
        body = "amount=%s" % amount
        response = await self.http_client.fetch(url, method="POST", body=body, auth_username=self.api_key,
                                                auth_password=self.api_secret, raise_error=False)
        return response


class _Refunds:
    def __init__(self, api_key, api_secret, client: AsyncHTTPClient):
        self.api_key = api_key
        self.api_secret = api_secret
        self.base_url = "https://api.razorpay.com/"
        self.version = "v1"
        self.payment_directory = "payments"
        self.api_url = "%s%s/%s/" % (self.base_url, self.version, self.payment_directory)
        self.http_client = client

    async def create(self, payment_id, data={}):
        url = "%s%s/refund" % (self.api_url, payment_id)
        body = ""
        if "amount" in data:
            body = "amount=%s" % data["amount"]
        response = await self.http_client.fetch(url, body=body, method="POST", auth_username=self.api_key,
                                                auth_password=self.api_secret, raise_error=False)
        return response

    async def all(self, payment_id, from_time=None, to_time=None, count=None, skip=None):
        args = _optional_args([["from", from_time], ["to", to_time], ["count", count], ["skip", skip]])
        url = "%s%s/refunds%s" % (self.api_url, payment_id, args)
        response = await self.http_client.fetch(url, auth_username=self.api_key, auth_password=self.api_secret,
                                                raise_error=False)
        return response

    async def fetch(self, payment_id, refund_id):
        url = "%s%s/refunds/%s" % (self.api_url, payment_id, refund_id)
        response = await self.http_client.fetch(url, auth_username=self.api_key, auth_password=self.api_secret,
                                                raise_error=False)
        return response


class Client:
    def __init__(self, api_key, api_secret):
        self.api_key = api_key
        self.api_secret = api_secret
        self.http_client = AsyncHTTPClient()
        self.payment = _Payments(api_key, api_secret, self.http_client)
        self.refund = _Refunds(api_key, api_secret, self.http_client)
